from src.netcdf_handler import NetCdfHandler
from src.models.database_interface import DatabaseInterface
from settings import DB_HOST, DB_PORT, DB_USERNAME, DB_PASSWORD, DB_DATABASE_NAME

from influxdb import DataFrameClient


class DatabaseSingleton(DatabaseInterface):

    # Do not access directly, use getter to ensure an instance is created
    _instance: DataFrameClient = None

    @staticmethod
    def get_instance(host=DB_HOST,
                     port=DB_PORT,
                     username=DB_USERNAME,
                     password=DB_PASSWORD,
                     database=DB_DATABASE_NAME
                     ) -> DataFrameClient:
        """
        Fetches or creates a new database instance
        :param host: Address to database, standard 'localhost'
        :param port: Port number for database, standard '8086'
        :param username: Username for database, standard 'root'
        :param password: Password for database, standard 'root'
        :param database: Database name
        :return: Database instance, if none exists then one will be created
        """

        if DatabaseSingleton._instance is None:
            DatabaseSingleton._instance = DataFrameClient(host=host,
                                                          port=port,
                                                          username=username,
                                                          password=password,
                                                          database=database,
                                                          )
        return DatabaseSingleton._instance

    def query(self, query):
        return self.get_instance().query(query)

    @staticmethod
    def upload_values(filepath: str, measurement_name: str):
        """
        Uploads the values of single netCDF file into a measurement
        :return: void
        """
        handler = NetCdfHandler()

        context = DatabaseSingleton.get_instance()
        context.create_database(DB_DATABASE_NAME)

        data = handler.netcdf_to_dataframe(filepath)
        context.write_points(data, measurement=measurement_name)

    # Uploads values to influxdb from file.
    @staticmethod
    def upload_all(directory_path: str, measurement: str) -> None:
        """
        Uploads the values of single netCDF file into a measurement
        :return: void
        """
        handler = NetCdfHandler()

        context = DatabaseSingleton.get_instance()
        context.create_database(DB_DATABASE_NAME)

        # Converts to csv and returns the new filepath
        file_list = handler.return_files(directory_path)
        for file in file_list:
            data = handler.netcdf_to_dataframe(file)
            context.write_points(data, measurement=measurement)
        return None


if __name__ == '__main__':
    context = DatabaseSingleton.get_instance()
    context.create_database("wow")

    result = context.query("SELECT * FROM testship;")
    print(result)
